import { Directive, ElementRef, Input, Output, EventEmitter, forwardRef, HostListener } from '@angular/core';
import { FlatpickrDefaults } from './flatpickr-defaults.service';
import * as flatpickr from 'flatpickr';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export var FLATPICKR_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return FlatpickrDirective; }),
    multi: true
};
var FlatpickrDirective = (function () {
    function FlatpickrDirective(elm, defaults) {
        this.elm = elm;
        this.defaults = defaults;
        /**
         * Gets triggered once the calendar is in a ready state
         */
        this.flatpickrReady = new EventEmitter();
        /**
         * Gets triggered when the user selects a date, or changes the time on a selected date.
         */
        this.flatpickrChange = new EventEmitter();
        /**
         * Gets triggered when the input value is updated with a new date string.
         */
        this.flatpickrValueUpdate = new EventEmitter();
        /**
         * Gets triggered when the calendar is opened.
         */
        this.flatpickrOpen = new EventEmitter();
        /**
         * Gets triggered when the calendar is closed.
         */
        this.flatpickrClose = new EventEmitter();
        /**
         * Gets triggered when the month is changed, either by the user or programmatically.
         */
        this.flatpickrMonthChange = new EventEmitter();
        /**
         * Gets triggered when the year is changed, either by the user or programmatically.
         */
        this.flatpickrYearChange = new EventEmitter();
        /**
         * Take full control of every date cell with this output
         */
        this.flatpickrDayCreate = new EventEmitter();
        this.onChangeFn = function () { }; // tslint:disable-line
    }
    FlatpickrDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        var options = {
            altFormat: this.altFormat,
            altInput: this.altInput,
            altInputClass: this.altInputClass,
            allowInput: this.allowInput,
            appendTo: this.appendTo,
            clickOpens: this.clickOpens,
            dateFormat: this.dateFormat,
            disable: this.disable,
            disableMobile: this.disableMobile,
            enable: this.enable,
            enableTime: this.enableTime,
            enableSeconds: this.enableSeconds,
            hourIncrement: this.hourIncrement,
            defaultDate: this.initialValue,
            inline: this.inline,
            maxDate: this.maxDate,
            minDate: this.minDate,
            minuteIncrement: this.minuteIncrement,
            mode: this.mode,
            nextArrow: this.nextArrow,
            noCalendar: this.noCalendar,
            parseDate: this.parseDate,
            prevArrow: this.prevArrow,
            shorthandCurrentMonth: this.shorthandCurrentMonth,
            static: this.static,
            time24hr: this.time24hr,
            weekNumbers: this.weekNumbers,
            getWeek: this.getWeek,
            wrap: this.wrap,
            plugins: this.plugins,
            onChange: function (selectedDates, dateString, instance) {
                _this.flatpickrChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onOpen: function (selectedDates, dateString, instance) {
                _this.flatpickrOpen.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onClose: function (selectedDates, dateString, instance) {
                _this.flatpickrClose.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onMonthChange: function (selectedDates, dateString, instance) {
                _this.flatpickrMonthChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onYearChange: function (selectedDates, dateString, instance) {
                _this.flatpickrYearChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onReady: function (selectedDates, dateString, instance) {
                _this.flatpickrReady.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onValueUpdate: function (selectedDates, dateString, instance) {
                _this.flatpickrValueUpdate.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onDayCreate: function (selectedDates, dateString, instance, dayElement) {
                _this.flatpickrDayCreate.emit({
                    selectedDates: selectedDates,
                    dateString: dateString,
                    instance: instance,
                    dayElement: dayElement
                });
            }
        };
        if (this.locale) {
            // workaround warning from flatpickr
            options.locale = this.locale;
        }
        Object.keys(options).forEach(function (key) {
            if (typeof options[key] === 'undefined') {
                options[key] = _this.defaults[key];
            }
        });
        options.time_24hr = options.time24hr;
        this.instance = flatpickr(this.elm.nativeElement, options);
    };
    FlatpickrDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (this.instance) {
            Object.keys(changes).forEach(function (inputKey) {
                _this.instance.set(inputKey, _this[inputKey]);
            });
        }
    };
    FlatpickrDirective.prototype.ngOnDestroy = function () {
        this.instance.destroy();
    };
    FlatpickrDirective.prototype.writeValue = function (value) {
        var convertedValue = value;
        if (this.convertModelValue && this.mode === 'range' && value) {
            convertedValue = [value.from, value.to];
        }
        if (this.instance) {
            this.instance.setDate(convertedValue);
        }
        else {
            // flatpickr hasn't been initialised yet, store the value for later use
            this.initialValue = convertedValue;
        }
    };
    FlatpickrDirective.prototype.registerOnChange = function (fn) {
        this.onChangeFn = fn;
    };
    FlatpickrDirective.prototype.registerOnTouched = function (fn) { }; // tslint:disable-line
    FlatpickrDirective.prototype.inputChanged = function () {
        var value = this.elm.nativeElement.value;
        if (this.convertModelValue && typeof value === 'string') {
            switch (this.mode) {
                case 'multiple':
                    var dates = value.split('; ').map(function (str) { return new Date(str); });
                    this.onChangeFn(dates);
                    break;
                case 'range':
                    var _a = value.split(' to ').map(function (str) { return new Date(str); }), from = _a[0], to = _a[1];
                    this.onChangeFn({ from: from, to: to });
                    break;
                case 'single':
                default:
                    this.onChangeFn(new Date(value));
            }
        }
        else {
            this.onChangeFn(value);
        }
    };
    FlatpickrDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[mwlFlatpickr]',
                    providers: [FLATPICKR_CONTROL_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    FlatpickrDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: FlatpickrDefaults, },
    ]; };
    FlatpickrDirective.propDecorators = {
        'altFormat': [{ type: Input },],
        'altInput': [{ type: Input },],
        'altInputClass': [{ type: Input },],
        'allowInput': [{ type: Input },],
        'appendTo': [{ type: Input },],
        'clickOpens': [{ type: Input },],
        'dateFormat': [{ type: Input },],
        'disable': [{ type: Input },],
        'disableMobile': [{ type: Input },],
        'enable': [{ type: Input },],
        'enableTime': [{ type: Input },],
        'enableSeconds': [{ type: Input },],
        'hourIncrement': [{ type: Input },],
        'inline': [{ type: Input },],
        'maxDate': [{ type: Input },],
        'minDate': [{ type: Input },],
        'minuteIncrement': [{ type: Input },],
        'mode': [{ type: Input },],
        'nextArrow': [{ type: Input },],
        'noCalendar': [{ type: Input },],
        'parseDate': [{ type: Input },],
        'prevArrow': [{ type: Input },],
        'shorthandCurrentMonth': [{ type: Input },],
        'static': [{ type: Input },],
        'time24hr': [{ type: Input },],
        'weekNumbers': [{ type: Input },],
        'getWeek': [{ type: Input },],
        'wrap': [{ type: Input },],
        'plugins': [{ type: Input },],
        'locale': [{ type: Input },],
        'convertModelValue': [{ type: Input },],
        'flatpickrReady': [{ type: Output },],
        'flatpickrChange': [{ type: Output },],
        'flatpickrValueUpdate': [{ type: Output },],
        'flatpickrOpen': [{ type: Output },],
        'flatpickrClose': [{ type: Output },],
        'flatpickrMonthChange': [{ type: Output },],
        'flatpickrYearChange': [{ type: Output },],
        'flatpickrDayCreate': [{ type: Output },],
        'inputChanged': [{ type: HostListener, args: ['input',] },],
    };
    return FlatpickrDirective;
}());
export { FlatpickrDirective };
//# sourceMappingURL=flatpickr.directive.js.map